<?php
/**
 * @Copyright
 * @package     Project CMS MD5 CHECKSUMS
 * @author      Viktor Vogel <admin@kubik-rubik.de>
 * @version     1.1.0 - 2016-05-17
 * @link        https://checksums.kubik-rubik.de/
 *
 * @license     GNU/GPL
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  See <http://www.gnu.org/licenses/>.
 *
 * How to use:
 *
 * Upload this PHP file to the root of your CMS installation.
 *
 * Call the script in your browser with 3 parameters:
 * - token - e.g. mLxJqCGR1NcdX8Ju (you have to specify this token also directly in the script)
 * - system - e.g. joomla, wordpress, drupal etc.
 * - version - e.g. x.y.z
 * http(s)://example.com/checksumscan.php?token=mLxJqCGR1NcdX8Ju&system=joomla&version=3.5.1
 *
 * Optional parameter:
 * - delete - Deletes the scan file automatically once scan process was executed
 * http(s)://example.com/checksumscan.php?token=mLxJqCGR1NcdX8Ju&system=joomla&version=3.5.1&delete=1
 * - core - Only show core files in the result list
 * http(s)://example.com/checksumscan.php?token=mLxJqCGR1NcdX8Ju&system=joomla&version=3.5.1&core=1
 *
 * You may also use all 5 parameters in one request!
 */

/**
 * Class ChecksumScan
 */
class ChecksumScan
{
	protected $system;
	protected $version;
	protected $delete = false;
	protected $core_only = false;
	protected $core_checksum_domain = 'https://checksums.kubik-rubik.de';
	protected $core_checksums = array();
	protected $system_checksums = array();
	protected $compare_result = array();
	protected $scan_result;
	protected $token;

	public function __construct()
	{
		// Define an individual security token here, e.g. $this->token = 'mLxJqCGR1NcdX8Ju';
		$this->token = '';

		if($this->preChecks())
		{
			$this->setConfiguration();
			$this->loadCoreChecksums();
			$this->loadSystemChecksums(__DIR__);
			$this->compareChecksums();
			$this->createScanResult();
			$this->printScanResult();
		}
	}

	/**
	 * Does basic pre-checks before executing the main process
	 *
	 * @return bool
	 */
	private function preChecks()
	{
		if(empty($_GET['token']) OR empty($this->token))
		{
			exit('Please specify a security token (as a GET parameter and in the scan file)! See <a href="https://checksums.kubik-rubik.de/" target="_blank" title="Project CMS MD5 CHECKSUMS">project page</a> for more information.');
		}

		$token = htmlspecialchars($_GET['token']);

		if($this->token != $token)
		{
			exit('Token is wrong! Please try again. See <a href="https://checksums.kubik-rubik.de/" target="_blank" title="Project CMS MD5 CHECKSUMS">project page</a> for more information.');
		}

		if(empty($_GET['system']))
		{
			exit('Please specify the system! See <a href="https://checksums.kubik-rubik.de/" target="_blank" title="Project CMS MD5 CHECKSUMS">project page</a> for more information.');
		}

		if(empty($_GET['version']))
		{
			exit('Please specify version! See <a href="https://checksums.kubik-rubik.de/" target="_blank" title="Project CMS MD5 CHECKSUMS">project page</a> for more information.');
		}

		if(!function_exists('curl_init'))
		{
			exit('cURL is required to load the core checksums! See <a href="https://checksums.kubik-rubik.de/" target="_blank" title="Project CMS MD5 CHECKSUMS">project page</a> for more information.');
		}

		return true;
	}

	/**
	 * Sets the configuration and needed parameters
	 */
	private function setConfiguration()
	{
		@ini_set('error_reporting', 0);
		@ini_set('memory_limit', '256M');
		@set_time_limit(3600);

		$this->system = htmlspecialchars($_GET['system']);
		$this->version = htmlspecialchars($_GET['version']);

		if(!empty($_GET['delete']))
		{
			$this->delete = (bool)htmlspecialchars($_GET['delete']);
		}

		if(!empty($_GET['core']))
		{
			$this->core_only = (bool)htmlspecialchars($_GET['core']);
		}
	}

	/**
	 * Loads the core checksums from the CMS MD5 CHECKSUMS project website
	 *
	 * @return bool
	 */
	private function loadCoreChecksums()
	{
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_URL, $this->core_checksum_domain.'/'.$this->system.'/'.$this->version);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);

		$core_checksums = json_decode(curl_exec($curl), true);
		$curl_error = curl_error($curl);
		curl_close($curl);

		if(!empty($curl_error))
		{
			exit($curl_error);
		}

		if(count($core_checksums) == 1 AND !empty($core_checksums['error']))
		{
			exit($core_checksums['error']);
		}

		$this->core_checksums = $core_checksums;

		return true;
	}

	/**
	 * Loads all checksums from the current system recursively
	 *
	 * @param  string $folder
	 * @param string  $folder_relative
	 *
	 * @return bool
	 */
	private function loadSystemChecksums($folder, $folder_relative = '')
	{
		if(!$dir = @opendir($folder))
		{
			return false;
		}

		while($file = readdir($dir))
		{
			if(is_dir($folder.'/'.$file) AND $file != '.' AND $file != '..')
			{
				if(!empty($this->exclude_folders))
				{
					if(in_array($folder_relative.$file, $this->exclude_folders))
					{
						continue;
					}
				}

				$this->loadSystemChecksums($folder.'/'.$file, $folder_relative.$file.'/');
			}
			elseif(is_file($folder.'/'.$file))
			{
				if(!empty($this->exclude_files))
				{
					if(in_array($folder_relative.$file, $this->exclude_files))
					{
						continue;
					}
				}

				$this->system_checksums[$folder_relative.$file] = md5_file($folder.'/'.$file);
			}
		}

		closedir($dir);

		return true;
	}

	/**
	 * Compares the core checksums with the checksums from the core
	 *
	 * @return bool
	 */
	private function compareChecksums()
	{
		$this->compare_result = array('modified' => array(), 'removed' => array(), 'new' => array(), 'identical' => array());

		if(!empty($this->core_checksums) AND !empty($this->system_checksums))
		{
			foreach($this->core_checksums as $core_key => $core_checksum)
			{
				if(!empty($this->system_checksums[$core_key]))
				{
					if($this->system_checksums[$core_key] == $core_checksum)
					{
						$this->compare_result['identical'][] = $core_key;
						unset($this->system_checksums[$core_key]);
						continue;
					}

					$this->compare_result['modified'][] = $core_key;
					unset($this->system_checksums[$core_key]);
					continue;
				}

				$this->compare_result['removed'][] = $core_key;
				unset($this->system_checksums[$core_key]);
				continue;
			}

			if(!empty($this->system_checksums) AND empty($this->core_only))
			{
				if($this->delete)
				{
					unset($this->system_checksums[basename(__FILE__)]);
				}

				foreach($this->system_checksums as $system_key => $system_checksum)
				{
					$this->compare_result['new'][] = $system_key;
					unset($this->system_checksums[$system_key]);
				}
			}
		}

		return true;
	}

	/**
	 * Creates the HTML of the scan result
	 *
	 * @return bool
	 */
	private function createScanResult()
	{
		$this->scan_result = 'Scan result could not be created properly, please try again!';

		if(!empty($this->compare_result))
		{
			$this->scan_result = '';

			foreach($this->compare_result as $key => $values)
			{
				$this->scan_result .= '<h2>'.ucfirst($key).' - '.count($values).' file(s)</h2>';

				if(!empty($values))
				{
					$this->scan_result .= '<ul>';

					foreach($values as $value)
					{
						$this->scan_result .= '<li>'.$value.'</li>';
					}

					$this->scan_result .= '</ul>';

					continue;
				}

				$this->scan_result .= '<ul><li>-</li></ul>';
			}
		}

		return true;
	}

	/**
	 * Prints out the result page and removes the script if set
	 */
	private function printScanResult()
	{
		echo '<html><head><title>Checksum Scan - CMS MD5 Checksums</title></head><body>';
		echo '<h1>Checksum Scan - CSM MD5 Checksums</h1>';
		echo '<p>Powered by the project <a href="https://checksums.kubik-rubik.de/" title="CMS MD5 CHECKSUMS">CMS MD5 CHECKSUMS</a></p>';
		echo '<p>Please support the CMS MD5 Checksums project with a donation: <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=6P8TGUJH2GU4S" target="_blank" title="Donation via Paypal">Paypal / Credit Card</a>. Thank you!</p>';
		echo $this->deleteScanFile();
		echo $this->scan_result;
		echo '</body></html>';
	}

	/**
	 * Deletes the script files once the process was executed if set
	 *
	 * @return string
	 */
	private function deleteScanFile()
	{
		if($this->delete)
		{
			if(unlink(__FILE__) == true)
			{
				return '<p>Checksum Scan file was deleted successfully!</p>';
			}

			return '<p>Checksum Scan file could not be deleted, please remove manually!</p>';
		}

		return '<p>Checksum Scan file was not deleted, please remove manually!</p>';
	}
}

new ChecksumScan();
